import { Controller, Get, Inject } from '@nestjs/common';
import {
  HealthCheckService,
  HealthCheck,
  HealthCheckResult,
} from '@nestjs/terminus';
import { PrismaHealthIndicatorUseCase } from '../../domain/usecases/prisma.health-indicator.usecase';
import { MESSAGE_PUBLISHER } from 'src/app.constants';
import { IPublishMessage } from 'src/interfaces/message-publisher';

@Controller('health')
export class HealthController {
  constructor(
    private readonly prismaHealthIndicatorUseCase: PrismaHealthIndicatorUseCase,
    private readonly healthCheckService: HealthCheckService,
    @Inject(MESSAGE_PUBLISHER)
    private readonly messagePublisher: IPublishMessage,
  ) {}

  @Get()
  @HealthCheck()
  async check() {
    try {
      return await this.healthCheckService.check([
        async () => this.prismaHealthIndicatorUseCase.isHealthy('prisma'),
      ]);
    } catch (error) {
      const healthCheckResult: HealthCheckResult = error.response;
      this.messagePublisher.publish(
        'logging.user.health.crit',
        JSON.stringify(healthCheckResult.error),
      );
      throw error;
    }
  }
}
