import { Module } from '@nestjs/common';
import { HealthServerController } from './adapters/primaries/health-server.controller';
import { PrismaHealthIndicatorUseCase } from './domain/usecases/prisma.health-indicator.usecase';
import { UsersRepository } from '../user/adapters/secondaries/users.repository';
import { DatabaseModule } from '../database/database.module';
import { HealthController } from './adapters/primaries/health.controller';
import { TerminusModule } from '@nestjs/terminus';
import { MESSAGE_BROKER_PUBLISHER, MESSAGE_PUBLISHER } from 'src/app.constants';
import { MessageBrokerPublisher } from '@mobicoop/message-broker-module';
import { MessagePublisher } from './adapters/secondaries/message-publisher';

@Module({
  imports: [TerminusModule, DatabaseModule],
  controllers: [HealthServerController, HealthController],
  providers: [
    PrismaHealthIndicatorUseCase,
    UsersRepository,
    {
      provide: MESSAGE_BROKER_PUBLISHER,
      useClass: MessageBrokerPublisher,
    },
    {
      provide: MESSAGE_PUBLISHER,
      useClass: MessagePublisher,
    },
  ],
})
export class HealthModule {}
