import { RedisClientOptions } from '@liaoliaots/nestjs-redis';
import { Module } from '@nestjs/common';
import { ConfigModule, ConfigService } from '@nestjs/config';
import { CqrsModule } from '@nestjs/cqrs';
import { redisStore } from 'cache-manager-ioredis-yet';
import { DatabaseModule } from '../database/database.module';
import { UserController } from './adapters/primaries/user.controller';
import { UsersRepository } from './adapters/secondaries/users.repository';
import { CreateUserUseCase } from './domain/usecases/create-user.usecase';
import { DeleteUserUseCase } from './domain/usecases/delete-user.usecase';
import { FindAllUsersUseCase } from './domain/usecases/find-all-users.usecase';
import { FindUserByUuidUseCase } from './domain/usecases/find-user-by-uuid.usecase';
import { UpdateUserUseCase } from './domain/usecases/update-user.usecase';
import { UserProfile } from './mappers/user.profile';
import { CacheModule } from '@nestjs/cache-manager';
import { MessageBrokerPublisher } from '@mobicoop/message-broker-module';
import {
  MESSAGE_BROKER_PUBLISHER,
  MESSAGE_PUBLISHER,
} from '../../app.constants';
import { MessagePublisher } from './adapters/secondaries/message-publisher';

@Module({
  imports: [
    DatabaseModule,
    CqrsModule,
    CacheModule.registerAsync<RedisClientOptions>({
      imports: [ConfigModule],
      useFactory: async (configService: ConfigService) => ({
        store: await redisStore({
          host: configService.get<string>('REDIS_HOST'),
          port: configService.get<number>('REDIS_PORT'),
          password: configService.get<string>('REDIS_PASSWORD'),
          ttl: configService.get('CACHE_TTL'),
        }),
      }),
      inject: [ConfigService],
    }),
  ],
  controllers: [UserController],
  providers: [
    UserProfile,
    UsersRepository,
    FindAllUsersUseCase,
    FindUserByUuidUseCase,
    CreateUserUseCase,
    UpdateUserUseCase,
    DeleteUserUseCase,
    {
      provide: MESSAGE_BROKER_PUBLISHER,
      useClass: MessageBrokerPublisher,
    },
    {
      provide: MESSAGE_PUBLISHER,
      useClass: MessagePublisher,
    },
  ],
  exports: [],
})
export class UserModule {}
